@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: AFPDocumentHandler.java 1891052 2021-06-26 06:39:20Z ssteiner $ */
+/* $Id$ */
 
 package org.apache.fop.render.afp;
 
@@ -164,7 +164,7 @@ public class AFPDocumentHandler extends AbstractBinaryWritingIFDocumentHandler
                     getPaintingState(), getDataStream(), getResourceManager());
         } else {
             return new AFPRectanglePainter(
-                    getPaintingState(), getDataStream());
+                    getPaintingState(), getDataStream(), getResourceManager());
         }
     }
 
@@ -216,7 +216,9 @@ public class AFPDocumentHandler extends AbstractBinaryWritingIFDocumentHandler
     public void startPageSequence(String id) throws IFException {
         try {
             if (!"false".equals(getContext().getForeignAttribute(AFPElementMapping.PAGE_GROUP))) {
-                dataStream.startPageGroup();
+                boolean addToPreviousPageGroup =
+                        "true".equals(getContext().getForeignAttribute(AFPElementMapping.ADD_TO_PREVIOUS_PAGE_GROUP));
+                dataStream.startPageGroup(!addToPreviousPageGroup);
             }
         } catch (IOException ioe) {
             throw new IFException("I/O error in startPageSequence()", ioe);
@@ -240,7 +242,7 @@ public class AFPDocumentHandler extends AbstractBinaryWritingIFDocumentHandler
             }
 
             //End page sequence
-            dataStream.endPageGroup();
+            dataStream.endPageGroup(false);
         } catch (IOException ioe) {
             throw new IFException("I/O error in endPageSequence()", ioe);
         }
@@ -476,6 +478,10 @@ public class AFPDocumentHandler extends AbstractBinaryWritingIFDocumentHandler
         paintingState.setNativeImagesSupported(nativeImages);
     }
 
+    public void setNativePDFImagesSupported(boolean nativeImages) {
+        paintingState.setNativePDFImagesSupported(nativeImages);
+    }
+
     /** {@inheritDoc} */
     public void setCMYKImagesSupported(boolean value) {
         paintingState.setCMYKImagesSupported(value);
@@ -491,6 +497,13 @@ public class AFPDocumentHandler extends AbstractBinaryWritingIFDocumentHandler
         this.paintingState.setBitmapEncodingQuality(quality);
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    public void setUseIocaImages(boolean useIocaImages) {
+        this.paintingState.setUseIocaImages(useIocaImages);
+    }
+
     /** {@inheritDoc} */
     public void setShadingMode(AFPShadingMode shadingMode) {
         this.shadingMode = shadingMode;
@@ -545,6 +558,10 @@ public class AFPDocumentHandler extends AbstractBinaryWritingIFDocumentHandler
         paintingState.setFS45(fs45);
     }
 
+    public void setMaskEnabled(boolean maskEnabled) {
+        paintingState.setMaskEnabled(maskEnabled);
+    }
+
     /** {@inheritDoc} */
     public boolean getWrapPSeg() {
         return  paintingState.getWrapPSeg();
