@@ -84,6 +84,9 @@ public class AFPRendererConfigurator extends PrintRendererConfigurator {
         if (config.isNativeImagesSupported() != null) {
             documentHandler.setNativeImagesSupported(config.isNativeImagesSupported());
         }
+        if (config.isNativePDFImagesSupported() != null) {
+            documentHandler.setNativePDFImagesSupported(config.isNativePDFImagesSupported());
+        }
         if (config.getShadingMode() != null) {
             documentHandler.setShadingMode(config.getShadingMode());
         }
@@ -108,6 +111,9 @@ public class AFPRendererConfigurator extends PrintRendererConfigurator {
         if (config.getBitmapEncodingQuality() != null) {
             documentHandler.setBitmapEncodingQuality(config.getBitmapEncodingQuality());
         }
+        if (config.isUseIocaImages() != null) {
+            documentHandler.setUseIocaImages(config.isUseIocaImages());
+        }
         if (config.getLineWidthCorrection() != null) {
             documentHandler.setLineWidthCorrection(config.getLineWidthCorrection());
         }
